/***************************************************************************
                          configloader.cpp  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <fstream>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <stdlib.h>
#include "configloader.h"
#include "functions.h"
#include "configdirectives.h"

using namespace lineak_util_functions;

ConfigLoader::ConfigLoader() {
}
ConfigLoader::ConfigLoader(string filename) : Loader(filename) {
}
ConfigLoader::~ConfigLoader() {
}
/** Process a config file with a single section */
map<const string, string>* ConfigLoader::processSingle(Vector* rawData) {
        if (rawData->empty())
    		return pnull;

       map<const string, string> *parsedData = new map<const string, string>;
       parsedData->clear();
       string tmpStore = null;
       string key=null;
       string data=null;
       int loc = 0;
       for(unsigned int i = 0; i < rawData->size(); i++) {
          	tmpStore = (*rawData)[i];
            loc = tmpStore.find('=');
            if (loc == 0) continue;
            key = strip_space(tmpStore.substr(0,loc));
            data = strip_space(tmpStore.substr(loc+1, tmpStore.size()));
            (*parsedData)[key] =  data;
       }
      delete (rawData);
      return parsedData;
}

/** Load the config for the file */
LConfig ConfigLoader::loadConfig() {
      return (*(getConfObj(processSingle(loadFile()))));
}
/** Load the config for the file filename */
LConfig ConfigLoader::loadConfig(string filename){
	LConfig tmp;
	if ( setFile(filename) )
		return (loadConfig());
	return	 tmp;
}
/** Load the parsed information and return a config object. */
LConfig* ConfigLoader::getConfObj(map<const string, string> *results){

    // If we don't have any values loaded. Return pnull
    if (results == pnull )
    	return (new LConfig);
    if (results->empty()) {
		delete (results);
		return (new LConfig);
	}

	ConfigDirectives d;
	d.addValue("KeyboardType", (*results)["KeyboardType"]);
	d.addValue("CdromDevice", (*results)["CdromDevice"]);
	d.addValue("MixerDevice", (*results)["MixerDevice"]);
	d.addValue("Screensaver", (*results)["Screensaver"]);
  /** Remove our entries */
	results->erase("KeyboardType");
	results->erase("CdromDevice");
	results->erase("MixerDevice");
	results->erase("Screensaver");
	
  #ifdef HAVE_XOSD
  d.addValue("XOSD_font", (((*results)["XOSD_font"] != "") ? (*results)["XOSD_font"] : XOSD_FONT));
  d.addValue("XOSD_color", (((*results)["XOSD_color"] != "") ? (*results)["XOSD_color"] : XOSD_COLOR));

  string position = (*results)["XOSD_pos"];
              if (position == "top")
                d.addValue("XOSD_pos", XOSD_top);
              if (position == "middle")
                d.addValue("XOSD_pos", XOSD_middle);
              if (position == "bottom" || position == "")
                d.addValue("XOSD_pos", XOSD_bottom);
  string align = (*results)["XOSD_align"];
              if (align == "left")
                d.addValue("XOSD_align", XOSD_left);
              if (align == "center" || align == "")
                d.addValue("XOSD_align", XOSD_center);
              if (align == "right")
                d.addValue("XOSD_align", XOSD_right);

  d.addValue("XOSD_timeout", (((*results)["XOSD_timeout"] != "") ? atoi((*results)["XOSD_timeout"].c_str()) : XOSD_TIMEOUT));
  d.addValue("XOSD_hoffset", (((*results)["XOSD_hoffset"] != "") ? atoi((*results)["XOSD_hoffset"].c_str()) : XOSD_HOFFSET));
  d.addValue("XOSD_voffset", (((*results)["XOSD_voffset"] != "") ? atoi((*results)["XOSD_voffset"].c_str()) : XOSD_VOFFSET));
  d.addValue("XOSD_soffset", (((*results)["XOSD_soffset"] != "") ? atoi((*results)["XOSD_soffset"].c_str()) : XOSD_SOFFSET));

	results->erase("XOSD_font");
	results->erase("XOSD_color");
	results->erase("XOSD_timeout");
	results->erase("XOSD_pos");
	results->erase("XOSD_align");
	results->erase("XOSD_hoffset");
	results->erase("XOSD_voffset");
	results->erase("XOSD_soffset");
  #endif

	// Initialize and fill in the values for the config object here.
	LConfig *conf = new LConfig(d);

	for (map<const string, string>::const_iterator i = results->begin(); i != results->end(); i++) {
      string command = i->second;
//      command = lineak_util_functions::strip(command,"\"#");
  	  conf->addKey(i->first, LCommand(command));
    }
    delete (results);
	return (conf);
}

LConfig & operator>>(ConfigLoader &ldr, LConfig &lconf) {
	return (lconf = ldr.loadConfig());
}

